import WebSocket from "ws";
import { UserContainer, UserGroup } from "./utils/user";
import { User, userIdType } from "./utils/user";
import SecurityManager from "./utils/secure";

export default class Manager {
    private security = new SecurityManager();
    private users = new UserContainer();
    private sessions = new Map<string, UserGroup>();
    private regions = new Map<string, UserGroup>();
    constructor(private serverName: string) {
        this.regions.set(this.serverName, new UserGroup(this.users));
    }

    generateId(): string {
        return SecurityManager.generateId();
    }

    createSession(name:string,
        maxUsers: number,
        sessionId: string = SecurityManager.generateId()): UserGroup {
        var group = new UserGroup(this.users, name, maxUsers);
        this.sessions.set(sessionId, group);
        return group;
    }

    getSession(sessionId: string) {
        return this.sessions.get(sessionId);
    }

    waitForSession(sessionId: string, userId: userIdType): boolean | undefined {
        if (this.addSession(sessionId, userId) === false) {
            return false;
        }
        let session = this.sessions.get(sessionId);
        if (session?.getNowUsers() !== session?.getMaxUsers()) {
            return undefined;
        }
        return true;
    }

    addSession(sessionId: string, userId: userIdType): boolean {
        let session = this.sessions.get(sessionId);
        if (session === undefined) {
            return false;
        }
        return session.addUser(userId);
    }

    removeSession(sessionId: string): boolean {
        return this.sessions.delete(sessionId);
    }

    generateUser(name: string, fingerprint: string, ws: WebSocket,
        userId: string = SecurityManager.generateId()) {
        const user = new User(name,
            this.security.generateToken(userId, fingerprint), ws);
        this.users.addUser(userId, user);
        return { userId, user };
    }

    getUser(id: userIdType) {
        return this.users.getUser(id);
    }

    addToRegion(id: userIdType, token: string, regionId: string): boolean {
        if (token === undefined || token === null) {
            return false;
        }
        if (this.security.verifyToken(token) == null) {
            return false;
        }
        return this.regions.get(regionId)?.addUser(id) !== undefined;
    }

    getIterRegion(regionId: string): IterableIterator<[userIdType,User]> | undefined {
        return this.regions.get(regionId)?.getUsersIterator();
    }

    getIterRegions(): IterableIterator<[string,UserGroup]> {
        return this.regions.entries();
    }

    removeUser(id: userIdType) {
        this.users.removeUser(id);
    }
}