import WebSocket from "ws";
import EventEmitter from "events";

export type userIdType = string;
type userNameType = string;
type userTokenType = string;

export class User {
    private name: userNameType;
    private token: userTokenType;
    private ws: WebSocket;
    
    constructor(name: userNameType, token: userTokenType, ws: WebSocket) {
        this.name = name;
        this.ws = ws;
        this.token = token;
    }

    dispose() {
        this.ws.close();
    }

    handleMessage(message: string) {
        console.log(`${this.getName()} 收到消息: ${message}`);
        // 这里可以是更复杂的逻辑处理消息
    }

    send(message: string) {
        this.ws.send(message);
    }

    getName(): userNameType {
        return this.name;
    }

    setName(name: userNameType) {
        this.name = name;
    }

    getToken(): userTokenType {
        return this.token;
    }

    getWs(): WebSocket {
        return this.ws;
    }
}

export class UserContainer extends EventEmitter {
    private users;
    constructor() {
        super();
        this.users = new Map<userIdType, User>();
    }

    addUser(id: userIdType ,user: User) {
        this.users.set(id, user);
    }

    removeUser(id: userIdType) {
        this.users.get(id)?.dispose();
        this.emit("userRemove", id);
        this.users.delete(id);
    }

    getUser(id: userIdType) {
        return this.users.get(id);
    }
}

export class UserGroup extends EventEmitter {
    private users;
    constructor(private userContainer: UserContainer,
            private name: string = "default",
            private maxUsers: number = 15,
            private garbageType: "auto" | "manual" = "auto") {
        super();
        this.users = new Map<userIdType, User>();
        userContainer.on('userRemove', this.removeUser.bind(this));
    };
    
    addUser(userId: userIdType): boolean {
        if (this.users.size === this.maxUsers) {
            console.error(`房间 ${this.name} 已满员，无法添加新用户。`);
            return false;
        }
        const user = this.userContainer.getUser(userId);
        
        if (user) {
            this.users.set(userId, user);
            return true;
        } else {
            console.error(`尝试添加的用户ID ${userId} 对应的用户实例不存在。`);
            return false;
        }
    }
    
    removeUser(user: userIdType): boolean {
       var res = this.users.delete(user);
       this.emit('GroupUserRemove', user, this.users.size);
       return res;
    }
    
    sendToAll(message: string) {
        for (const user of this.users.values()) {
            user.send(message);
        }
    }
    
    setName(name: string) {
        this.name = name;
    }

    getName() {
        return this.name;
    }

    setMaxUsers(maxUsers: number) {
        this.maxUsers = maxUsers;
    }

    getMaxUsers() {
        return this.maxUsers;
    }

    getNowUsers() {
        return this.users.size;
    }
    
    sendToAllExcept(message: string, excludeUserId?: userIdType) {
        for (const [userId, user] of this.users.entries()) {
            if (excludeUserId !== userId) {
                user.send(message);
            }
        }
    }

    sendTo(message: string, userId: userIdType) {
        const user = this.users.get(userId);
        if (user) {
            user.send(message);
        }
    }

    getUsersIterator(): IterableIterator<[userIdType, User]> {
        return this.users.entries();
    }

    hasUser(userId: userIdType):boolean {
        return this.users.has(userId);
    }
}